@@ -6,11 +6,13 @@ from typing import Set
 import orjson
 
 PBT_CONFIG_FILE_NAME = "pbtconfig.json"
+PBT_LOCK_FILE_NAME = "pbt.lock"
 
 
 @dataclass
 class PBTConfig:
     cwd: Path
+    cache_dir: Path
     ignore_packages: Set[str]
 
     @staticmethod
@@ -39,6 +41,12 @@ class PBTConfig:
             )
 
         cwd = Path(cwd)
+        cache_dir = cwd / ".cache"
+        cache_dir.mkdir(exist_ok=True, parents=True)
+
         with open(str(cwd / PBT_CONFIG_FILE_NAME), "r") as f:
             cfg = orjson.loads(f.read())
-        return PBTConfig(cwd=cwd, ignore_packages=set(cfg["ignore_packages"]))
+
+        return PBTConfig(
+            cwd=cwd, ignore_packages=set(cfg["ignore_packages"]), cache_dir=cache_dir
+        )
