@@ -1,16 +1,20 @@
+import os
 import shutil
 import subprocess
 from dataclasses import dataclass
 from enum import Enum
 from glob import glob
+from graphlib import TopologicalSorter
 from pathlib import Path
-from typing import Dict, Union, List
+from typing import Dict, Optional, Union, List
 
 import orjson
 import toml
 from loguru import logger
 
 from pbt.config import PBTConfig
+from pbt.diff import is_modified
+from pbt.git import Git
 from pbt.poetry import Poetry
 
 
@@ -30,28 +34,33 @@ class Package:
     # list of packages that use the current package
     invert_inter_dependencies: List["Package"]
 
-    def build(self):
-        """Build the package"""
-        outfile = f"{self.name}-{self.version}.tar.gz"
-        rebuild = True
-        if (self.dir / "dist" / outfile).exists():
-            if not self.is_modified():
-                rebuild = False
+    def build(self, cfg: PBTConfig):
+        """Build the package if needed"""
+        # check if package has been modified since the last built
+        whl_file = self.get_wheel_file()
+        if whl_file is not None:
+            if not self.is_modified(cfg):
+                logger.info("Skip package {} as the content does not change", self.name)
+                return
 
-        if rebuild:
-            logger.info("Build package {}", self.name)
-            if (self.dir / "dist").exists():
-                shutil.rmtree(str(self.dir / "dist"))
-            subprocess.check_call(["poetry", "build"], cwd=str(self.dir))
+        logger.info("Build package {}", self.name)
+        if (self.dir / "dist").exists():
+            shutil.rmtree(str(self.dir / "dist"))
+        subprocess.check_call(["poetry", "build"], cwd=str(self.dir))
 
-    def is_modified(self) -> bool:
-        """Whether the package has been modified since the last version"""
-        return True
+    def compute_pip_hash(self, cfg: PBTConfig, build: bool = True) -> str:
+        """Compute hash of the content of the package"""
+        if build:
+            self.build(cfg)
+        output = subprocess.check_output(["pip", "hash", self.get_wheel_file()]).decode().strip()
+        output = output[output.find("--hash=")+len("--hash="):]
+        assert output.startswith("sha256:")
+        return output[len("sha256:"):]
 
-    def install_into(self, package: "Package", build: bool = False):
+    def install_into(self, package: "Package", cfg: PBTConfig, build: bool = True):
         """Install the current package (self) in the virtual environment of another package (`package`)"""
         if build:
-            self.build()
+            self.build(cfg)
         logger.info(
             "Install package {} into package {} environment", self.name, package.name
         )
@@ -64,13 +73,23 @@ class Package:
         assert package.name in self.dependencies
         self.pkg_handler.update_package(self, package.name, package.version)
 
+    def is_modified(self, cfg: PBTConfig):
+        """Check if package has been modified"""
+        return is_modified(self, cfg)
+
     @property
     def pkg_handler(self):
         if self.type == PackageType.Poetry:
             return Poetry
         raise NotImplementedError(self.type)
+    
+    def get_tar_file(self) -> Optional[str]:
+        tar_file = self.dir / "dist" / f"{self.name}-{self.version}.tar.gz"
+        if tar_file.exists():
+            return str(tar_file)
+        return None
 
-    def get_wheel_file(self):
+    def get_wheel_file(self) -> Optional[str]:
         whl_files = glob(str(self.dir / f"dist/{self.name.replace('-', '_')}*.whl"))
         if len(whl_files) == 0:
             return None
@@ -153,3 +172,13 @@ def search_packages(pbt_cfg: PBTConfig):
             packages[pname].invert_inter_dependencies.append(package)
 
     return packages
+
+
+def topological_sort(packages: Dict[str, Package]) -> List[str]:
+    """Sort the packages so that the first item is always leaf node in the dependency graph (i.e., it doesn't use any
+    package in the repository.
+    """
+    graph = {}
+    for package in packages.values():
+        graph[package.name] = {child.name for child in package.inter_dependencies}
+    return list(TopologicalSorter(graph).static_order())
